@@ -2,7 +2,7 @@ import { Config } from '@stencil/core';
 
 export const config: Config = {
   namespace: 'dashboard',
-  buildEs5: true,
+  buildEs5: false,
   outputTargets: [
     {
       type: 'dist',
@@ -14,6 +14,13 @@ export const config: Config = {
     {
       type: 'docs-readme',
     },
+    {
+      // copy wasm
+      type: 'dist',
+      copy: [
+        { src: '../node_modules/@quakeworks/quake_wasm/quake_wasm_bg.wasm', dest: 'quake_wasm_bg.wasm' }
+      ]
+    },
     {
       type: 'www',
       serviceWorker: null, // disable service workers
