@@ -6,13 +6,10 @@ use quake_core::QuakeConfig;
 pub mod entry_action;
 pub mod helper;
 pub mod quake_action;
-pub mod transflow_action;
 
 pub fn action(expr: QuakeActionNode, conf: QuakeConfig) -> Result<(), Box<dyn Error>> {
     match expr.object.as_str() {
         "quake" => quake_action::quake_action(expr.action, &conf),
-        "flow" => transflow_action::transflow_action(expr.action, &conf),
-        "transflow" => Ok(()),
         _ => entry_action::entry_action(&expr, conf),
     }
 }
