@@ -1,10 +1,13 @@
 use std::error::Error;
-use std::fs;
-use std::path::PathBuf;
+use std::io::{self, Cursor};
+use std::path::{Path, PathBuf};
+use std::{env, fs};
 
 use clap::Parser;
+use futures::executor::block_on;
 use futures::future;
 use helper::entry_watcher;
+use pagedump::page_dump;
 use tracing::{debug, error};
 
 use quake_core::entry::entry_defines::EntryDefines;
@@ -16,10 +19,14 @@ use crate::server::quake_rocket;
 
 pub mod cli;
 pub mod helper;
+pub mod pagedump;
 pub mod server;
+pub mod usecases;
+
+const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[derive(Parser)]
-#[clap(version = "0.1.2", author = "Inherd <quake@inherd.org>")]
+#[clap(version = "0.3.0", author = "Inherd <quake@inherd.org>")]
 pub struct Opts {
     #[clap(subcommand)]
     cmd: SubCommand,
@@ -35,11 +42,22 @@ pub enum SubCommand {
     Server(WebServer),
     /// terminal UI
     Tui(Terminal),
+    /// dump page for web deploy
+    Pagedump(PageDump),
 }
 
 #[derive(Parser)]
 pub struct Terminal {}
 
+#[derive(Parser)]
+pub struct PageDump {
+    #[clap(short, long, default_value = ".quake.yaml")]
+    config: String,
+
+    #[clap(short, long, default_value = "output")]
+    output: String,
+}
+
 #[derive(Parser)]
 pub struct WebServer {
     #[clap(short, long, default_value = ".quake.yaml")]
@@ -55,6 +73,10 @@ pub struct Init {
     /// init by path
     #[clap(short, long, default_value = ".")]
     path: String,
+
+    #[clap(short, long)]
+    /// download web.zip from GitHub
+    download: bool,
 }
 
 #[derive(Parser, Debug)]
@@ -82,11 +104,11 @@ async fn main() {
 
 pub async fn process_cmd(opts: Opts) -> Result<(), Box<dyn Error>> {
     match opts.cmd {
-        SubCommand::Init(init) => init_projects(init)?,
+        SubCommand::Init(init) => init_projects(init).await?,
         SubCommand::Cmd(cmd) => {
             let conf = config_quake(&cmd)?;
-            if cmd.input.len() > 0 {
-                let expr = QuakeActionNode::action_from_text(cmd.input.as_str())?;
+            if !cmd.input.is_empty() {
+                let expr = QuakeActionNode::from_text(cmd.input.as_str())?;
                 cli::action(expr, conf)?
             }
         }
@@ -96,7 +118,7 @@ pub async fn process_cmd(opts: Opts) -> Result<(), Box<dyn Error>> {
             let search_url = config.search_url;
 
             if server.watch {
-                futures::executor::block_on(async {
+                block_on(async {
                     let (_s, _g) = future::join(
                         quake_rocket().launch(),
                         entry_watcher::async_watch(path, search_url),
@@ -104,12 +126,17 @@ pub async fn process_cmd(opts: Opts) -> Result<(), Box<dyn Error>> {
                     .await;
                 });
             } else {
-                let _ = futures::executor::block_on(async { quake_rocket().launch() }).await;
+                #[allow(clippy::async_yields_async)]
+                let _ = block_on(async { quake_rocket().launch() }).await;
             }
         }
         SubCommand::Tui(_) => {
             tui_main_loop()?;
         }
+        SubCommand::Pagedump(dump) => {
+            let config = load_config(&dump.config)?;
+            page_dump(config);
+        }
     }
 
     Ok(())
@@ -136,7 +163,7 @@ fn config_quake(cmd: &Command) -> Result<QuakeConfig, Box<dyn Error>> {
     Ok(conf)
 }
 
-fn load_config(path: &String) -> Result<QuakeConfig, Box<dyn Error>> {
+fn load_config(path: &str) -> Result<QuakeConfig, Box<dyn Error>> {
     let content =
         fs::read_to_string(path).expect("lost .quake.yaml config, please run `quake init`");
     let conf: QuakeConfig = serde_yaml::from_str(content.as_str()).expect("serde .quake.yml error");
@@ -144,27 +171,28 @@ fn load_config(path: &String) -> Result<QuakeConfig, Box<dyn Error>> {
     Ok(conf)
 }
 
-fn init_projects(config: Init) -> Result<(), Box<dyn Error>> {
+async fn init_projects(config: Init) -> Result<(), Box<dyn Error>> {
     fs::create_dir_all(&config.path)?;
 
-    let path = PathBuf::from(&config.path).join(".quake.yaml");
-    let define = PathBuf::from(&config.path).join("entries-define.yaml");
+    let workspace = PathBuf::from(&config.path);
+    let path = workspace.join(".quake.yaml");
+    let define = workspace.join("entries-define.yaml");
 
-    let config = QuakeConfig {
-        workspace: config.path.clone(),
+    let quake_config = QuakeConfig {
+        workspace: config.path,
         editor: "".to_string(),
         search_url: "http://127.0.0.1:7700".to_string(),
         server_location: "web".to_string(),
         port: 8000,
     };
 
-    fs::write(&path, serde_yaml::to_string(&config)?)?;
+    fs::write(&path, serde_yaml::to_string(&quake_config)?)?;
     debug!("create .quake.yaml in {:?}", &path.display());
 
     let todo_define = "
 - type: todo
   display: Todo
-  fields:
+  properties:
     - title: Title
     - author: String
 ";
@@ -176,6 +204,66 @@ fn init_projects(config: Init) -> Result<(), Box<dyn Error>> {
     fs::write(&define, serde_yaml::to_string(&file)?)?;
     debug!("create default entry defines in {:?}", &define.display());
 
+    if config.download {
+        let target = format!(
+            "https://github.com/phodal/quake/releases/download/v{}/web.zip",
+            VERSION
+        );
+        debug!("download web.zip from {}", target);
+
+        let response = reqwest::get(&target).await?.bytes().await?;
+        unzip_all(Cursor::new(response.to_vec()), &workspace)?;
+    }
+
+    Ok(())
+}
+
+fn unzip_all(reader: Cursor<Vec<u8>>, plugins_path: &Path) -> Result<(), Box<dyn Error>> {
+    let mut archive = zip::ZipArchive::new(reader)?;
+
+    for i in 0..archive.len() {
+        let mut file = archive.by_index(i)?;
+        let out_path = match file.enclosed_name() {
+            Some(path) => plugins_path.join(path),
+            None => continue,
+        };
+
+        {
+            let comment = file.comment();
+            if !comment.is_empty() {
+                println!("Plugin {} comment: {}", i, comment);
+            }
+        }
+
+        if (&*file.name()).ends_with('/') {
+            println!("File {} extracted to \"{}\"", i, out_path.display());
+            fs::create_dir_all(&out_path)?;
+        } else {
+            println!(
+                "File {} extracted to \"{}\" ({} bytes)",
+                i,
+                out_path.display(),
+                file.size()
+            );
+            if let Some(p) = out_path.parent() {
+                if !p.exists() {
+                    fs::create_dir_all(&p)?;
+                }
+            }
+            let mut outfile = fs::File::create(&out_path)?;
+            io::copy(&mut file, &mut outfile)?;
+        }
+
+        // Get and Set permissions
+        #[cfg(unix)]
+        {
+            use std::os::unix::fs::PermissionsExt;
+
+            if let Some(mode) = file.unix_mode() {
+                fs::set_permissions(&out_path, fs::Permissions::from_mode(mode))?;
+            }
+        }
+    }
     Ok(())
 }
 
@@ -226,6 +314,7 @@ mod tests {
             process_cmd(Opts {
                 cmd: SubCommand::Init(Init {
                     path: test_dir.to_string(),
+                    download: false,
                 }),
             })
             .await
@@ -242,21 +331,36 @@ mod tests {
                 &"water".to_string(),
             );
 
-            let content = fs::read_to_string(paths.base.join("0001-samples.md")).unwrap();
+            let content = fs::read_to_string(paths.entry_path.join("0001-samples.md")).unwrap();
             let file = EntryFile::from(content.as_str(), 1).unwrap();
 
-            let title = file.field("title");
+            let title = file.property("title");
             assert_eq!(title.unwrap(), "samples");
 
             fs::remove_dir_all(test_dir).unwrap();
         });
     }
 
+    #[test]
+    #[ignore = "need tokio runtime"]
+    fn should_download_webapp_dist() {
+        task::block_on(async {
+            let test_dir = "test_dir";
+            process_cmd(Opts {
+                cmd: SubCommand::Init(Init {
+                    path: test_dir.to_string(),
+                    download: true,
+                }),
+            })
+            .await
+            .unwrap();
+        })
+    }
+
     fn config_dir() -> PathBuf {
-        let conf_dir = PathBuf::from("_fixtures")
+        PathBuf::from("_fixtures")
             .join("configs")
-            .join(".quake.yaml");
-        conf_dir
+            .join(".quake.yaml")
     }
 
     #[ignore]
